﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using Ellab.Charting;

namespace ChartingSandbox
{
    public partial class ChartingSandboxForm : Form
    {
        private const int NO_OF_CHART_POINTS = 50;
        private double[] x = new double[NO_OF_CHART_POINTS];
        private double[] y = new double[NO_OF_CHART_POINTS];
        private DrawLine drawLine = null;
        private List<DrawLine> drawLineList = new List<DrawLine>();
        private ISeries _serie;
        private EventHandler DrawLineSelectEventHandler;
        private EventHandler DrawLineDraggedEventHandler;
        private EventHandler DrawLineNewEventHandler;
        private DateTime _startTime = DateTime.Now;


        public ChartingSandboxForm()
        {
            InitializeComponent();
            FillChartControl("irrelevant1");

            chartControl.MouseDown += new MouseEventHandler(chartControl_MouseDown);
        }

        private void FillChartControl(string unit)
        {
            Random randomY = new Random();
            DateTime roundedDateTime = new DateTime(_startTime.Year, _startTime.Month, _startTime.Day, _startTime.Hour, _startTime.Minute, _startTime.Second);

            for (int i = 0; i < NO_OF_CHART_POINTS; i++)
            {
                x[i] = roundedDateTime.AddSeconds((double)i).ToOADate();
                y[i] = (double)randomY.Next(100);
            }
            _serie = chartControl.CreateSeries(Ellab.Charting.SeriesType.LineSeries, unit);
            _serie.Color = Color.Red;
            _serie.Data.Add(x, y);
        }

        void chartControl_MouseDown(object sender, MouseEventArgs e)
        {
            //Steema.TeeChart.Tools.DrawLineItem drawlineItems = (chartControl.TChart.Tools[0] as Steema.TeeChart.Tools.DrawLine).Clicked(e.X, e.Y);
            //if (drawlineItems != null)
            //{
            //    if (drawlineItems.StartHandle.Contains(e.X, e.Y))
            //    {
            //        this.Text = "Start Handle X value: " + (DateTime.FromOADate(drawlineItems.StartPos.X)).ToShortDateString() + "Start Handle Y value: " + ((int)drawlineItems.StartPos.Y).ToString();

            //    }
            //    else if (drawlineItems.EndHandle.Contains(e.X, e.Y))
            //    {
            //        this.Text = "Start Handle X value: " + (DateTime.FromOADate(drawlineItems.EndPos.X)).ToShortDateString() + "Start Handle Y value: " + ((int)drawlineItems.EndPos.Y).ToString();
            //    }
            //}
        }

        private void buttonShowData_Click(object sender, EventArgs e)
        {
            ShowData.ShowDataGridView showDataGridView = new ShowData.ShowDataGridView();
            for (int i = 0; i < NO_OF_CHART_POINTS; i++)
            {
                showDataGridView.AddData(x[i], y[i]);
            }
            showDataGridView.Show();
        }

        private void buttonAddDrawLine_Click(object sender, EventArgs e)
        {
            try
            {
                drawLine = (DrawLine)chartControl.CreateTool(ToolType.DrawLine);
                drawLine.Width = 10;
                drawLine.Serie = _serie;

                if (DrawLineSelectEventHandler == null)
                {
                    DrawLineSelectEventHandler = new EventHandler(drawLine_DrawLineSelectEvent);
                }
                drawLine.DrawLineSelectEvent += DrawLineSelectEventHandler;
                
                if (DrawLineDraggedEventHandler == null)
                {
                    DrawLineDraggedEventHandler = new EventHandler(drawLine_DrawLineDraggedEvent);
                }
                drawLine.DrawLineDraggedEvent += DrawLineDraggedEventHandler;
                
                if (DrawLineNewEventHandler == null)
                {
                    DrawLineNewEventHandler = new EventHandler(drawLine_DrawLineDraggedNew);
                }
                drawLine.DrawLineNew += DrawLineNewEventHandler;
            }
            catch
            { }
        }

        private int _counter = 0;
        void drawLine_DrawLineDraggedNew(object sender, EventArgs e)
        {
            drawLine = (DrawLine)sender;
            drawLineList.Add(drawLine);
            drawLine.Text = "Noget " + _counter;
            _counter++;
        }

        void drawLine_DrawLineDraggedEvent(object sender, EventArgs e)
        {
            drawLine = (DrawLine)sender;
        }

        void drawLine_DrawLineSelectEvent(object sender, EventArgs e)
        {
            drawLine = (DrawLine)sender;
        }

        private void buttonDeleteLine_Click(object sender, EventArgs e)
        {
            drawLine.DeleteSelected();
        }

        internal void buttonMakeZoom_Click(object sender, EventArgs e)
        {
            double startTimePlus5 = _startTime.AddSeconds(5).ToOADate();
            double startTimePlus25 = _startTime.AddSeconds(25).ToOADate();

            //Either like this:...
            //Ellab.Charting.MirrorTool mirrorTool = (Ellab.Charting.MirrorTool)chartControl.CreateTool(ToolType.MirrorTool);
            //mirrorTool.SetSeries(_serie.Type, _serie.Unit);
            //mirrorTool.VisibleMirrorRectangle.SetVisibleRectangle(startTimePlus5, startTimePlus25, 5, 25);

            //byte[] byteArrayImage = mirrorTool.GetMirror();

            //...or like this:
            byte[] byteArrayImage = Ellab.Charting.MirrorTool.GetMirror(this.chartControl, _serie, startTimePlus5, startTimePlus25, 5, 25);

            MirrorForm mirrorForm = new MirrorForm();
            Image returnImage = null;
            using (MemoryStream ms = new MemoryStream(byteArrayImage))
            {
                returnImage = Image.FromStream(ms);
            }
            mirrorForm.pictureBox1.Image = returnImage;
            mirrorForm.Show();
        }
    }
}
