﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace Ellab.Charting
{
    public class DrawLine : ITool, IDisposable
    {
        private Steema.TeeChart.Tools.DrawLine tDrawLine;

        private Steema.TeeChart.Tools.DrawLineSelectingEventHandler _steemaTeeChartToolsDrawLineSelectingEventHandlerSelecting = null;
        private Steema.TeeChart.Tools.DrawLineEventHandler _steemaTeeChartToolsDrawLineEventHandlerSelect = null;
        private Steema.TeeChart.Tools.DrawLineEventHandler _steemaTeeChartToolsDrawLineEventHandlerDrag = null;
        private Steema.TeeChart.Tools.DrawLineEventHandler _steemaTeeChartToolsDrawLineEventHandlerDragged = null;
        private Steema.TeeChart.Tools.DrawLineEventHandler _steemaTeeChartToolsDrawLineEventHandlerNew = null;

        #region Properties

        public Chart Chart
        {
            get;
            private set;
        }

        private Annotation _annotation = null;
        private Annotation Annotation
        {
            get
            {
                if (_annotation == null)
                {
                    _annotation = Annotation.Create(Chart, this.Name + "Annotation");
                }
                return _annotation;
            }
        }

        private ISeries _serie;
        /// <summary>
        /// The Serie to whitch this DrawLine tool is to be assigned
        /// </summary>
        public ISeries Serie
        {
            get
            {
                if (_serie == null)
                {
                    //Ellab.Debug.GlobalDebug.debugForm.WriteText("Charting.MarksTip: Serie used before it has been set", Debug.DebugType.TeeChart);
                }
                return _serie;
            }
            set
            {
                tDrawLine.Series = value.baseSeries;
                _serie = value;
            }
        }

        public int Width
        {
            get { return tDrawLine.Pen.Width; }
            set { tDrawLine.Pen.Width = value; }
        }

        public System.Drawing.Point FromPoint
        {
            get { return tDrawLine.FromPoint; }
            set { tDrawLine.FromPoint = value; }
        }

        public System.Drawing.Point ToPoint
        {
            get { return tDrawLine.ToPoint; }
            set { tDrawLine.ToPoint = value; }
        }

        public int Transparency
        {
            get { return tDrawLine.Pen.Transparency; }
            set { tDrawLine.Pen.Transparency = value; }
        }

        public System.Drawing.Color Color
        {
            get { return tDrawLine.Pen.Color; }
            set { tDrawLine.Pen.Color = value; }
        }

        public string Text
        {
            get { return Annotation.Text; }
            set { Annotation.Text = value; }
        }

        public bool Visible
        {
            get { return tDrawLine.Pen.Visible; }
            set { tDrawLine.Pen.Visible = value; }
        }

        #endregion Properties

        internal DrawLine(Chart chart, string name, ISeries serie)
        {
            Chart = chart;
            Name = name;
            tDrawLine = new Steema.TeeChart.Tools.DrawLine(chart.TChart.Chart);

            tDrawLine.Style = Steema.TeeChart.Tools.DrawLineStyle.Line;
            tDrawLine.Pen.EndCap = System.Drawing.Drawing2D.LineCap.ArrowAnchor;
            System.Drawing.Pen pen = new System.Drawing.Pen(System.Drawing.Color.Green);
            tDrawLine.Pen.Width = 0;

            if (serie != null)
            {
                Serie = serie;
            }

            if (_steemaTeeChartToolsDrawLineEventHandlerNew == null)
            {
                _steemaTeeChartToolsDrawLineEventHandlerNew = new Steema.TeeChart.Tools.DrawLineEventHandler(tDrawLine_NewLine);
                tDrawLine.NewLine += _steemaTeeChartToolsDrawLineEventHandlerNew;
            }

            if (_steemaTeeChartToolsDrawLineSelectingEventHandlerSelecting == null)
            {
                _steemaTeeChartToolsDrawLineSelectingEventHandlerSelecting = new Steema.TeeChart.Tools.DrawLineSelectingEventHandler(tDrawLine_Selecting);
                tDrawLine.Selecting += _steemaTeeChartToolsDrawLineSelectingEventHandlerSelecting;
            }

            if (_steemaTeeChartToolsDrawLineEventHandlerSelect == null)
            {
                _steemaTeeChartToolsDrawLineEventHandlerSelect = new Steema.TeeChart.Tools.DrawLineEventHandler(tDrawLine_Select);
                tDrawLine.Select += _steemaTeeChartToolsDrawLineEventHandlerSelect;
            }

            if (_steemaTeeChartToolsDrawLineEventHandlerDrag == null)
            {
                _steemaTeeChartToolsDrawLineEventHandlerDrag = new Steema.TeeChart.Tools.DrawLineEventHandler(tDrawLine_DragLine);
                tDrawLine.DragLine += _steemaTeeChartToolsDrawLineEventHandlerDrag;
            }

            if (_steemaTeeChartToolsDrawLineEventHandlerDragged == null)
            {
                _steemaTeeChartToolsDrawLineEventHandlerDragged = new Steema.TeeChart.Tools.DrawLineEventHandler(tDrawLine_DraggedLine);
                tDrawLine.DraggedLine += _steemaTeeChartToolsDrawLineEventHandlerDragged;
            }

            Chart.AddTool(this);

            Chart.TChart.AfterDraw += new Steema.TeeChart.PaintChartEventHandler(TChart_AfterDraw);

            Chart.TChart.MouseClick += new System.Windows.Forms.MouseEventHandler(Chart_MouseClick);
        }

        void TChart_AfterDraw(object sender, Steema.TeeChart.Drawing.Graphics3D g)
        {
            Annotation.PixelPosition = tDrawLine.FromPoint;
        }

        void Chart_MouseClick(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            //if (Chart.TChart.Tools[2] is Steema.TeeChart.Tools.DrawLine)
            //{
            //    Steema.TeeChart.Tools.DrawLine myDrawLine = (Steema.TeeChart.Tools.DrawLine)Chart.TChart.Tools[2];
            //    Steema.TeeChart.Tools.DrawLineItem drawlineItems = (myDrawLine).Clicked(e.X, e.Y);
            //    if (drawlineItems != null)
            //    {
            //        if (drawlineItems.StartHandle.Contains(e.X, e.Y))
            //        {
            //            this.Text = "Start Handle X value: " + (DateTime.FromOADate(drawlineItems.StartPos.X)).ToShortDateString() + "Start Handle Y value: " + ((int)drawlineItems.StartPos.Y).ToString();

            //        }
            //        else if (drawlineItems.EndHandle.Contains(e.X, e.Y))
            //        {
            //            this.Text = "Start Handle X value: " + (DateTime.FromOADate(drawlineItems.EndPos.X)).ToShortDateString() + "Start Handle Y value: " + ((int)drawlineItems.EndPos.Y).ToString();
            //        }
            //    }
            //}
        }

        /// <summary>
        /// While creating a DrawLine tool will set most of the default values, it is not assined to a specific series,
        /// thus will operate on all series in the given chart. Other DrawLine tools might have been assined to a serie,
        /// potentially confusing things up. Therefor when usign this constructor, remember to assign a serie to this DrawLine 
        /// tool, using property 'public ISeries Serie'.
        /// </summary>
        /// <param name="chart"></param>
        /// <param name="name"></param>
        /// <returns></returns>
        public static DrawLine Create(Chart chart, string name)
        {
            return Create(chart, name, null);
        }

        public static DrawLine Create(Chart chart, string name, ISeries serie)
        {
            return new DrawLine(chart, name, serie);
        }

        public void DeleteSelected()
        {
            tDrawLine.DeleteSelected();
            Annotation.Dispose();
        }

        #region Events

        public event EventHandler DrawLineDraggedEvent;
        public event EventHandler DrawLineSelectEvent;
        public event EventHandler DrawLineNew;

        void tDrawLine_NewLine(Steema.TeeChart.Tools.DrawLine sender)
        {
            if (DrawLineNew != null)
            {
                DrawLineNew(this, new EventArgs());
            }
            Annotation.PixelPosition = tDrawLine.FromPoint;
            sender.EnableDraw = false;
            //int i = 0;
            //drawlineItems.StartHandle.Offset(5,5);//= new System.Drawing.Size(0, 0);
            //tDrawLine.Se
        }

        void tDrawLine_DragLine(Steema.TeeChart.Tools.DrawLine sender)
        {
            Annotation.PixelPosition = tDrawLine.FromPoint;
        }

        void tDrawLine_DraggedLine(Steema.TeeChart.Tools.DrawLine sender)
        {
            if (DrawLineDraggedEvent != null)
            {
                DrawLineDraggedEvent(this, new EventArgs());
            }
            Annotation.PixelPosition = tDrawLine.FromPoint;
        }

        void tDrawLine_Selecting(Steema.TeeChart.Tools.DrawLine sender, Steema.TeeChart.Tools.DrawLineItem line, ref bool allow)
        {
        }

        void tDrawLine_Select(Steema.TeeChart.Tools.DrawLine sender)
        {
            if (DrawLineSelectEvent != null)
            {
                DrawLineSelectEvent(this, new EventArgs());
            }
        }

        #endregion Events

        #region ITool Members

        public void Remove()
        {
            if (removeDelegate == null)
                removeDelegate = new Action(RemoveInternal);
            Chart.InvokeDelegate(removeDelegate);
        }
        private Action removeDelegate = null;
        private void RemoveInternal()
        {
            Chart.TChart.Tools.Remove(tDrawLine);
            Chart.RemoveTool(this);
        }

        public ToolType Type
        {
            get { return ToolType.DrawLine; }
        }

        public string Name { get; set; }

        public ITool CreateMirror(Chart chart)
        {
            DrawLine drawLine = new DrawLine(Chart, Name, Serie);
            return drawLine;
        }

        #endregion

        #region IDisposable Members

        public void Dispose()
        {
            //Shouldn be necessary, but never the less
            tDrawLine.NewLine -= _steemaTeeChartToolsDrawLineEventHandlerNew;
            tDrawLine.Selecting -= _steemaTeeChartToolsDrawLineSelectingEventHandlerSelecting;
            tDrawLine.Select -= _steemaTeeChartToolsDrawLineEventHandlerSelect;
            tDrawLine.DragLine -= _steemaTeeChartToolsDrawLineEventHandlerDrag;
            tDrawLine.DraggedLine -= _steemaTeeChartToolsDrawLineEventHandlerDragged;

            _steemaTeeChartToolsDrawLineSelectingEventHandlerSelecting = null;
            _steemaTeeChartToolsDrawLineEventHandlerSelect = null;
            _steemaTeeChartToolsDrawLineEventHandlerDrag = null;
            _steemaTeeChartToolsDrawLineEventHandlerDragged = null;
            _steemaTeeChartToolsDrawLineEventHandlerNew = null;

            Remove();
            tDrawLine = null;
        }

        #endregion

        /// <summary>
        /// DO NOT USE
        /// </summary>
        private DrawLine()
        {
            throw new NotImplementedException();
        }
    }
}
