﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace Ellab.Charting
{
    public class MirrorTool : ITool
    {
        public class VisibleRectangle
        {
            private Chart _parent = null;

            #region Properties

            private double _timeMinimum = double.NaN;
            public double TimeMinimum
            {
                get;
                set;
            }
            private double _timeMaximum = double.NaN;
            public double TimeMaximum
            {
                get;
                set;
            }

            public double ValueMinimum { get; set; }
            public double ValueMaximum { get; set; }

            #endregion Properties

            public VisibleRectangle(Chart parent)
            {
                _parent = parent;
            }

            public void SetVisibleRectangle(double timeMinimum, double timeMaximum, double valueMinimum, double valueMiximum)
            {
                TimeMinimum = timeMinimum;
                TimeMaximum = timeMaximum;
                ValueMinimum = valueMinimum;
                ValueMaximum = valueMiximum;
            }

            public Rectangle? GetRectangle()
            {
                if (!double.IsNaN(_timeMaximum) && !double.IsNaN(_timeMaximum))
                {
                    int x = _parent.HorizontalAxis.ValueToPixelCoordinate(TimeMinimum);
                    int y = _parent.VerticalAxes[0].ValueToPixelCoordinate(ValueMaximum);
                    int width = _parent.HorizontalAxis.ValueToPixelCoordinate(TimeMaximum) - _parent.HorizontalAxis.ValueToPixelCoordinate(TimeMinimum);
                    int height = _parent.VerticalAxes[0].ValueToPixelCoordinate(ValueMinimum) - _parent.VerticalAxes[0].ValueToPixelCoordinate(ValueMaximum);

                    return new Rectangle(x, y, width, height);
                }
                else
                {
                    return null;
                }
            }
        }

        private MirrorControl _mirrorControl = null;
        private Chart _parent;
        private static List<string> Names = new List<string>();

        #region Properties

        private VisibleRectangle _visibleMirrorRectangle = null;
        public VisibleRectangle VisibleMirrorRectangle 
        {
            get
            {
                if (_visibleMirrorRectangle == null)
                {
                    _visibleMirrorRectangle = new VisibleRectangle(_mirrorControl.chartControl);
                }
                return _visibleMirrorRectangle;
            }
            set
            {
                _visibleMirrorRectangle = value;
            }
        }

        #endregion Properties

        internal MirrorTool(Chart chart, string name)
        {
            _parent = chart;
            Name = name;

            _mirrorControl = new MirrorControl();
            _mirrorControl.chartControl.Title = _parent.Title;

            _parent.AddTool(this);
        }

        public void SetSeries(SeriesType type, string unit)
        {
            if (_mirrorControl != null)
            {
                foreach (ISeries serie in _parent.Series)
                {
                    if (type == serie.Type && unit == serie.Unit)
                    {
                        _mirrorControl.AddSerie(serie);
                        break;
                    }
                }
            }
        }

        public byte[] GetMirror()
        {
            Ellab.Charting.Chart.ChartZoom.ZoomRectangleCoordinates zoomRectangle = new Chart.ChartZoom.ZoomRectangleCoordinates();
            zoomRectangle.HorizontalAxesMaximum = this.VisibleMirrorRectangle.TimeMaximum;
            zoomRectangle.HorizontalAxesMinimum = this.VisibleMirrorRectangle.TimeMinimum;
            zoomRectangle.VerticalAxesMaximum = this.VisibleMirrorRectangle.ValueMaximum;
            zoomRectangle.VerticalAxesMinimum = this.VisibleMirrorRectangle.ValueMinimum;

            _mirrorControl.chartControl.Zoom.SetZoomCoordinates(zoomRectangle);
            _mirrorControl.chartControl.Zoom.DoCustomZoom();

            _mirrorControl.chartControl.Export.Height = 380;
            _mirrorControl.chartControl.Export.Width = 700;

            return _mirrorControl.chartControl.Export.ToArray();
        }

        public static MirrorTool Create(Chart chart, string name)
        {
            return new MirrorTool(chart, name);
        }

        public static byte[] GetMirror(Chart chart, ISeries serie, double timeMinimum, double timeMaximum, double valueMinimum, double ValueMaximum)
        { 
            string newName = "Mirror" + (Names.Count + 1);
            MirrorTool mirrorTool = new MirrorTool(chart, newName);
            mirrorTool.SetSeries(serie.Type, serie.Unit);
            mirrorTool.VisibleMirrorRectangle.SetVisibleRectangle(timeMinimum, timeMaximum, valueMinimum, ValueMaximum);

            return mirrorTool.GetMirror();

        }

        #region ITool Members

        public void Remove()
        {
            throw new NotImplementedException();
            //if (removeDelegate == null)
            //    removeDelegate = new Action(RemoveInternal);
            //_chart.InvokeDelegate(removeDelegate);
        }
        //private Action removeDelegate = null;
        //private void RemoveInternal()
        //{
        //    _chart.TChart.Tools.Remove(tChart);
        //    _chart.RemoveTool(this);
        //}


        public ToolType Type
        {
            get { return ToolType.MirrorTool; }
        }

        public string Name { get; set; }

        public ITool CreateMirror(Chart chart)
        {
            throw new NotImplementedException();
        }

        #endregion
    }
}
